@@ -29,6 +29,23 @@ class DexyHandler(object):
             if hasattr(self, 'EXECUTABLE'):
                 return self.EXECUTABLE
 
+    @classmethod
+    def executable_present(self):
+        """Determine whether the specified executable is available."""
+        if self.executable():
+            cmd = self.executable().split()[0]
+            which = "which %s" % cmd
+            proc = subprocess.Popen(which, shell=True,
+                                    stdout=subprocess.PIPE,
+                                    stderr=subprocess.PIPE)
+            proc.communicate()
+            if proc.returncode == 0:
+                return True
+            else:
+                return False
+        else:
+            return True
+
     @classmethod
     def version_command(self):
         if platform.system() == 'Windows':
@@ -54,11 +71,11 @@ class DexyHandler(object):
                     log.debug(err_msg)
                 else:
                     print err_msg
-                return "unspecified"
+                return "error"
             else:
                 return output
         else:
-            return "unspecified"
+            return None
 
     @classmethod
     def output_file_extension(klass, ext, key, next_handler_class):
